#include "DeerRender/Mesh.h"
#include "Deer/Log.h"
#include "DeerRender/Render/Buffer.h"

namespace Deer{
    Ref<VertexArray> Mesh::loadModelToGPU(const MeshData& data) {
        Ref<VertexArray> vertexArray = VertexArray::create();
        vertexArray->bind();

        Ref<VertexBuffer> vertexPositionBuffer = VertexBuffer::create(data.vertexPositionsData, data.vertexCount * sizeof(VertexPosition));

        BufferLayout positionLayout({
            {"v_position", DataType::Short3, ShaderDataType::FloatingPoint}
        }, sizeof(VertexPosition)); // 
        BufferLayout normalLayout({
            {"v_normal", DataType::Byte3, ShaderDataType::NormalizedFloatingPoint}
        }, sizeof(VertexNormal));

        vertexPositionBuffer->bind();
        vertexPositionBuffer->setLayout(positionLayout);
        vertexArray->addVertexBuffer(vertexPositionBuffer);

        Ref<VertexBuffer> vertexNormalBuffer = VertexBuffer::create(data.vertexNormalData, data.vertexCount * sizeof(VertexNormal));
        vertexNormalBuffer->bind();
        vertexNormalBuffer->setLayout(normalLayout);
        vertexArray->addVertexBuffer(vertexNormalBuffer);

        Ref<IndexBuffer> indexBuffer = IndexBuffer::create(data.indexData, data.indexCount * sizeof(uint16_t), IndexDataType::Unsigned_Short);
        indexBuffer->bind();
        vertexArray->setIndexBuffer(indexBuffer);

        return vertexArray;
    }
}