#include "MeshExplorer.h"

#include "Deer/DataStore.h"
#include "Deer/Path.h"
#include "DeerStudio/Editor/EditorUtils.h"
#include "DeerStudio/Editor/Icons.h"
#include "imgui.h"

namespace Deer {
	namespace MeshExplorer {
		Path m_meshExplorerPath(DEER_NULL_PATH);
		void drawFolder(const Path& path);
	}  // namespace MeshExplorer

	void MeshExplorer::onImGui() {
		if (m_meshExplorerPath == DEER_NULL_PATH) {
			DataStore::createFolder(DataStore::rootPath / DEER_MESH_PATH);
			m_meshExplorerPath = DEER_MESH_PATH;
		}

		ImGui::Begin("Mesh Explorer", (bool*)0);  // ImGuiWindowFlags_MenuBar

		ImGui::TextColored(ImVec4(0.5f, 0.5f, 0.5f, 1.0f), "%s",
		                   m_meshExplorerPath.generic_string().c_str());

		setupColumns(ICON_MIN_SIZE + 80);

		if (m_meshExplorerPath != DEER_MESH_PATH) {
			drawFolder("..");

			float cursorOffset =
			    (ICON_MIN_SIZE - ImGui::CalcTextSize("..").x) / 2;
			ImGui::SetCursorPos(ImVec2(cursorOffset + ImGui::GetCursorPos().x,
			                           ImGui::GetCursorPos().y));
			ImGui::Text("..");

			ImGui::NextColumn();
		}

		for (const auto& entry : std::filesystem::directory_iterator(
		         DataStore::rootPath / m_meshExplorerPath)) {
			if (entry.is_directory())
				drawFolder(entry.path());
			else {
			}

			float cursorOffset =
			    (ICON_MIN_SIZE -
			     ImGui::CalcTextSize(entry.path().stem().string().c_str()).x) /
			    2;
			ImGui::SetCursorPos(ImVec2(cursorOffset + ImGui::GetCursorPos().x,
			                           ImGui::GetCursorPos().y));
			ImGui::Text("%s", entry.path().stem().string().c_str());

			ImGui::NextColumn();
		}
		ImGui::Columns();
		ImGui::End();
	}

	void MeshExplorer::drawFolder(const Path& path) {
		ImGui::Image((void*)(uint64_t)Icons::folder_icon->getTextureID(),
		             ImVec2(ICON_MIN_SIZE, ICON_MIN_SIZE), ImVec2(0, 1),
		             ImVec2(1, 0));

		if (ImGui::IsItemClicked(0) && ImGui::IsMouseDoubleClicked(0)) {
			if (path == "..")
				m_meshExplorerPath = m_meshExplorerPath.parent_path();
			else
				m_meshExplorerPath = path;
		}
	}
}  // namespace Deer