#include "TreePannel.h"

#include "Deer/Enviroment.h"
#include "Deer/Scene.h"
#include "DeerRender/Input.h"
#include "DeerStudio/Editor/ActiveEntity.h"
#include "DeerStudio/Project.h"
#include "imgui.h"

namespace Deer {
	namespace TreePannel {
		void updateEntity(Entity& entity);
		void updateReciveDragPayload(Entity& entity);
		bool updateDragPayload(Entity* entity, const std::string& name);
		void updateContextMenu();
		void clickEntity(Entity& entity);

		bool m_isRightClickHandled;
		Entity* m_contextMenuEntity = nullptr;
	}  // namespace TreePannel

	void TreePannel::onImgui() {
		ImGui::Begin("Tree Pannel", (bool*)0, ImGuiWindowFlags_MenuBar);

		m_isRightClickHandled = false;
		Entity& root = Project::m_scene.getMainEnviroment().getRoot();

		if (!m_contextMenuEntity) m_contextMenuEntity = &root;

		if (ImGui::BeginMenuBar()) {
			if (ImGui::BeginMenu("New")) {
				if (ImGui::MenuItem("Entity")) {
					Entity& entity =
					    Project::m_scene.getMainEnviroment().createEntity(
					        "new entity");
					entity.setParent(root);
				}
				if (ImGui::MenuItem("Camera")) {
					Entity& entity =
					    Project::m_scene.getMainEnviroment().createEntity(
					        "new camera");
					entity.addComponent<CameraComponent>();
					entity.setParent(root);
				}
				ImGui::EndMenu();
			}
			ImGui::EndMenuBar();
		}

		ImGui::PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(0, 0));
		RelationshipComponent& relation =
		    Project::m_scene.getMainEnviroment()
		        .getRoot()
		        .getComponent<RelationshipComponent>();

		updateEntity(Project::m_scene.getMainEnviroment().getRoot());
		// for (int i = 0; i < relation.childCount; i++) {
		//	Entity& childEntity =
		//	    Project::m_scene.getMainEnviroment().getEntity(
		//	        relation.getChildrenId(i));
		// }
		ImGui::PopStyleVar();

		updateContextMenu();

		ImGui::Spacing();
		ImVec2 spaceSize(ImGui::GetWindowContentRegionWidth(), 80);

		ImGui::InvisibleButton("DragDropSpace", spaceSize);
		updateReciveDragPayload(root);

		if (!m_isRightClickHandled &&
		    ImGui::IsWindowHovered(
		        ImGuiHoveredFlags_AllowWhenBlockedByPopup |
		        ImGuiHoveredFlags_AllowWhenBlockedByActiveItem) &&
		    ImGui::IsMouseClicked(ImGuiMouseButton_Right) &&
		    !ImGui::IsAnyItemHovered()) {
			m_contextMenuEntity = &root;
			ImGui::OpenPopup("Entity Context Menu");
		}

		updateReciveDragPayload(root);
		ImGui::End();
	}

	void TreePannel::updateEntity(Entity& entity) {
		auto& tag = entity.getComponent<TagComponent>();
		auto& relationship = entity.getComponent<RelationshipComponent>();

		std::string name = (tag.tag == "") ? "-" : tag.tag;

		const void* entityID = reinterpret_cast<const void*>(
		    static_cast<uintptr_t>(entity.getId()));

		ImGui::Spacing();

		// End of the tree
		if (relationship.childCount == 0 && !entity.isRoot()) {
			ImGuiTreeNodeFlags flags = ImGuiTreeNodeFlags_Leaf |
			                           ImGuiTreeNodeFlags_NoTreePushOnOpen |
			                           ImGuiTreeNodeFlags_SpanFullWidth;
			if (ActiveEntity::contains(entity))
				flags |= ImGuiTreeNodeFlags_Selected;

			ImGui::TreeNodeEx(entityID, flags, "%s", name.c_str());
			if (!updateDragPayload(&entity, tag.tag))
				updateReciveDragPayload(entity);

			clickEntity(entity);

			if (ImGui::IsItemClicked(ImGuiMouseButton_Right)) {
				m_contextMenuEntity = &entity;
				m_isRightClickHandled = true;
				ImGui::OpenPopup("Entity Context Menu");
			}

			return;
		}

		ImGuiTreeNodeFlags flags = ImGuiTreeNodeFlags_OpenOnDoubleClick |
		                           ImGuiTreeNodeFlags_OpenOnArrow |
		                           ImGuiTreeNodeFlags_SpanFullWidth;
		if (ActiveEntity::contains(entity))
			flags |= ImGuiTreeNodeFlags_Selected;

		// for the moment i prefer to default open all
		flags |= ImGuiTreeNodeFlags_DefaultOpen;

		if (ImGui::TreeNodeEx(entityID, flags, "%s", name.c_str())) {
			if (!entity.isRoot()) updateDragPayload(&entity, tag.tag);
			updateReciveDragPayload(entity);

			if (ImGui::IsItemClicked(ImGuiMouseButton_Right)) {
				m_contextMenuEntity = &entity;
				m_isRightClickHandled = true;
				ImGui::OpenPopup("Entity Context Menu");
			}

			clickEntity(entity);

			RelationshipComponent& rc =
			    entity.getComponent<RelationshipComponent>();
			for (int i = 0; i < rc.childCount; i++) {
				uint16_t childID = rc.getChildrenId(i);
				Entity& childEntity =
				    m_contextMenuEntity->getEnvironment()->getEntity(childID);
				updateEntity(childEntity);
			}
			updateContextMenu();
			ImGui::TreePop();
		} else {
			clickEntity(entity);

			if (ImGui::IsItemClicked(ImGuiMouseButton_Right)) {
				m_contextMenuEntity = &entity;
				m_isRightClickHandled = true;
				ImGui::OpenPopup("Entity Context Menu");
			}

			if (!entity.isRoot()) updateDragPayload(&entity, tag.tag);
			updateReciveDragPayload(entity);
		}
	}

	void TreePannel::updateReciveDragPayload(Entity& entity) {
		if (ImGui::BeginDragDropTarget()) {
			if (const ImGuiPayload* payload =
			        ImGui::AcceptDragDropPayload("_ENTITY")) {
				Entity* receivedData = *(Entity**)payload->Data;

				if (!entity.isDescendantOf(*receivedData))
					receivedData->setParent(entity);
			}

			ImGui::EndDragDropTarget();
		}
	}

	bool TreePannel::updateDragPayload(Entity* entity,
	                                   const std::string& name) {
		if (!ImGui::BeginDragDropSource()) return false;

		ImGui::SetDragDropPayload("_ENTITY", &entity, sizeof(Entity*));

		ImGui::Text("%s", name.c_str());
		ImGui::EndDragDropSource();
		return true;
	}

	void TreePannel::clickEntity(Entity& entity) {
		if (ImGui::IsItemClicked() && !ImGui::IsItemToggledOpen()) {
			if (!(Input::isKeyPressed(DEER_KEY_LEFT_CONTROL) ||
			      Input::isKeyPressed(DEER_KEY_LEFT_ALT)))
				ActiveEntity::clear();

			if (Input::isKeyPressed(DEER_KEY_LEFT_ALT))
				ActiveEntity::removeEntity(entity);
			else
				ActiveEntity::addEntity(entity);
		}
	}

	void TreePannel::updateContextMenu() {
		bool callRename = false;

		if (ImGui::BeginPopup("Entity Context Menu")) {
			if (ImGui::MenuItem("New Entity")) {
				Entity& entity =
				    Project::m_scene.getMainEnviroment().createEntity(
				        "new entity");
				entity.setParent(*m_contextMenuEntity);

				ImGui::CloseCurrentPopup();
			}
			if (ImGui::MenuItem("New Camera")) {
				Entity& entity =
				    Project::m_scene.getMainEnviroment().createEntity(
				        "new camera");
				entity.addComponent<CameraComponent>();
				entity.setParent(*m_contextMenuEntity);

				ImGui::CloseCurrentPopup();
			}
			if (!m_contextMenuEntity->isRoot() && ImGui::MenuItem("Delete")) {
				m_contextMenuEntity->destroy();
				ActiveEntity::clear();
				ImGui::CloseCurrentPopup();
			}
			if (!m_contextMenuEntity->isRoot() && ImGui::MenuItem("Rename")) {
				callRename = true;
				ImGui::CloseCurrentPopup();
			}
			if (!m_contextMenuEntity->isRoot() &&
			    ImGui::MenuItem("Duplicate")) {
				m_contextMenuEntity->duplicate();
				ImGui::CloseCurrentPopup();
			}
			ImGui::EndPopup();
		}

		if (callRename) ImGui::OpenPopup("Rename Entity Menu");

		if (ImGui::BeginPopup("Rename Entity Menu")) {
			std::string& name =
			    m_contextMenuEntity->getComponent<TagComponent>().tag;
			char nameBuffer[256];
			std::strncpy(nameBuffer, name.c_str(), sizeof(nameBuffer) - 1);

			ImGui::Text("Rename");
			if (ImGui::InputText("##", nameBuffer, 256,
			                     ImGuiInputTextFlags_EnterReturnsTrue)) {
				m_contextMenuEntity->getComponent<TagComponent>().tag =
				    nameBuffer;
				ImGui::CloseCurrentPopup();
			}

			ImGui::EndPopup();
		}
	}
}  // namespace Deer